(* Script to check an md5 checksum

ToDo:

- escape quotes in filenames, because that will result in an error
- do the fileCount over all, not only every one selected folder (multiple dialogs)
- optional crc using "cksum" instead of md5 (cksum only on Snow Leopard (10.6) and maybe higher)

*)

(* Global Vars *)
global fileCount, logMessage, dialogTooMuchFilesMessage01, dialogTooMuchFilesMessage02, dialogTooMuchFilesYes, dialogTooMuchFilesNo, dialogNothingSelectedMessage, dialogNothingSelectedButton, dialogErrorOccurredMessage, dialogErrorOccurredButton, dialogChecksumFileMissingMessage01, dialogChecksumFileMissingMessage02, dialogChecksumFileMissingButton, dialogErrorWord, dialogAttentionWord, dialogChecksumMismatchMessage01, dialogChecksumMismatchMessage02


(* Properties (English) *)
property dialogAttentionWordEnglish : "ATTENTION! "
property dialogErrorWordEnglish : "Error: "
property dialogTooMuchFilesMessage01English : "Are you sure you want to check checksums for "
property dialogTooMuchFilesMessage02English : " files?"
property dialogTooMuchFilesYesEnglish : "Yes, sure!"
property dialogTooMuchFilesNoEnglish : "Hell, no!"
property dialogNothingSelectedMessageEnglish : "Please select one or more files in Finder."
property dialogNothingSelectedButtonEnglish : "Allright, I do whatever you say!"
property dialogErrorOccurredMessageEnglish : "One or more errors occured:" & return & return
property dialogErrorOccurredButtonEnglish : "Doh!"
property dialogChecksumFileMissingMessage01English : "The checksum file for the file \""
property dialogChecksumFileMissingMessage02English : "\" is missing or corrupted!"
property dialogChecksumFileMissingButtonEnglish : "Gee!"
property dialogChecksumMismatchMessage01English : "The checksum for the file \""
property dialogChecksumMismatchMessage02English : "\" does not match!"
property dialogChecksumsMatchMessageEnglish : "The checksums match!"
property dialogChecksumsMatchButtonEnglish : "Great!"

#property dialogChecksumMismatchButtonEnglish : "Gee!"
(* Properties (Deutsch) *)
property dialogAttentionWordGerman : "Achtung! "
property dialogErrorWordGerman : "Error: "
property dialogTooMuchFilesMessage01German : "Bist Du sicher, dass Du fr "
property dialogTooMuchFilesMessage02German : " Prfsummen berprfen willst?"
property dialogTooMuchFilesYesGerman : "Na klar!"
property dialogTooMuchFilesNoGerman : "Bloss nicht!"
property dialogNothingSelectedMessageGerman : "Bitte whle eine oder mehrere Dateien im Finder aus."
property dialogNothingSelectedButtonGerman : "Okay, mach ich!"
property dialogErrorOccurredMessageGerman : "Ein oder mehr Fehler ist aufgetreten:" & return & return
property dialogErrorOccurredButtonGerman : "Och n!"
property dialogChecksumFileMissingMessage01German : "Die Prfsummendatei fr die Datei \""
property dialogChecksumFileMissingMessage02German : "\" fehlt oder ist fehlerhaft!"
property dialogChecksumFileMissingButtonGerman : "Mist!"
property dialogChecksumMismatchMessage01German : "Die Prfsumme fr die Datei \""
property dialogChecksumMismatchMessage02German : "\" stimmt nicht berein!"
property dialogChecksumsMatchMessageGerman : "Die Prfsummen stimmen berein!"
property dialogChecksumsMatchButtonGerman : "Prima!"
#property dialogChecksumMismatchButtonGerman : "Gee!"

(* UnMain() *)
on run (argv)
	(* Init *)
	set logMessage to ""
	if (my systemLanguage() is "de") then
		set dialogAttentionWord to dialogAttentionWordGerman
		set dialogErrorWord to dialogErrorWordGerman
		set dialogTooMuchFilesMessage01 to dialogTooMuchFilesMessage01German
		set dialogTooMuchFilesMessage02 to dialogTooMuchFilesMessage02German
		set dialogTooMuchFilesYes to dialogTooMuchFilesYesGerman
		set dialogTooMuchFilesNo to dialogTooMuchFilesNoGerman
		set dialogNothingSelectedMessage to dialogNothingSelectedMessageGerman
		set dialogNothingSelectedButton to dialogNothingSelectedButtonGerman
		set dialogErrorOccurredMessage to dialogErrorOccurredMessageGerman
		set dialogErrorOccurredButton to dialogErrorOccurredButtonGerman
		set dialogChecksumFileMissingMessage02 to dialogChecksumFileMissingMessage01German
		set dialogChecksumFileMissingMessage02 to dialogChecksumFileMissingMessage02German
		set dialogChecksumFileMissingButton to dialogChecksumFileMissingButtonGerman
		set dialogChecksumMismatchMessage01 to dialogChecksumMismatchMessage01German
		set dialogChecksumMismatchMessage02 to dialogChecksumMismatchMessage02German
		set dialogChecksumsMatchMessage to dialogChecksumsMatchMessageGerman
		set dialogChecksumsMatchButton to dialogChecksumsMatchButtonGerman
	else
		set dialogAttentionWord to dialogAttentionWordEnglish
		set dialogErrorWord to dialogErrorWordEnglish
		set dialogTooMuchFilesMessage01 to dialogTooMuchFilesMessage01English
		set dialogTooMuchFilesMessage02 to dialogTooMuchFilesMessage02English
		set dialogTooMuchFilesYes to dialogTooMuchFilesYesEnglish
		set dialogTooMuchFilesNo to dialogTooMuchFilesNoEnglish
		set dialogNothingSelectedMessage to dialogNothingSelectedMessageEnglish
		set dialogNothingSelectedButton to dialogNothingSelectedButtonEnglish
		set dialogErrorOccurredMessage to dialogErrorOccurredMessageEnglish
		set dialogErrorOccurredButton to dialogErrorOccurredButtonEnglish
		set dialogChecksumFileMissingMessage01 to dialogChecksumFileMissingMessage01English
		set dialogChecksumFileMissingMessage02 to dialogChecksumFileMissingMessage02English
		set dialogChecksumFileMissingButton to dialogChecksumFileMissingButtonEnglish
		set dialogChecksumMismatchMessage01 to dialogChecksumMismatchMessage01English
		set dialogChecksumMismatchMessage02 to dialogChecksumMismatchMessage02English
		set dialogChecksumsMatchMessage to dialogChecksumsMatchMessageEnglish
		set dialogChecksumsMatchButton to dialogChecksumsMatchButtonEnglish
	end if
	
	(* Get selected files/folders *)
	tell application "Finder"
	activate
		set selectedItems to selection
		(* Check wether something selected *)
		if ((count of selectedItems) is greater than 0) then
			(* walk through items *)
			repeat with oneItem in selectedItems
				(* Do dry-run to determine file count *)
				set fileCount to 0
				tell me to checkChecksums(oneItem, true)
				log "fileCount: " & fileCount
				if (fileCount is greater than 23) then
					(* Ask user if he's sure that's what he wants to do *)
					display dialog dialogTooMuchFilesMessage01 & fileCount & dialogTooMuchFilesMessage02 buttons {dialogTooMuchFilesYes, dialogTooMuchFilesNo} default button 2
					set returnedButton to button returned of result
					if (returnedButton is not dialogTooMuchFilesYes) then
						tell me to exit repeat #quit
					end if
				end if
				(* Now actually create the checksums *)
				set fileCount to 0
				tell me to checkChecksums(oneItem, false)
			end repeat (* loop through sel. files *)
			(* If something went wrong -> tell the user *)
			if ((count of characters of logMessage) is greater than 0) then
				display dialog dialogErrorOccurredMessage & logMessage buttons {dialogErrorOccurredButton} default button 1 with icon 0 (* icon 0: stop sign, icon 1: App-Icon, icon 2: Att. Sign and App Icon *)
			else
				display dialog dialogChecksumsMatchMessage buttons {dialogChecksumsMatchButton} default button 1
			end if
		else
			display dialog dialogNothingSelectedMessage buttons dialogNothingSelectedButton
		end if
	end tell (* tell finder *)
end run

(* subroutine for checking md5 checksums *)
on checkChecksums(finderItem, dryRun)
	try
		(* check wether it's a file or a folder, if file create an additional md5 checksum file with similar name *)
		tell application "Finder"
			set theItemClass to class of finderItem
			set theItemName to name of finderItem
			set theItemContainer to (container of finderItem) as alias
			set itemPath to POSIX path of (finderItem as alias)
			set itemContainerPath to POSIX path of theItemContainer
			if (theItemClass is folder) then (* not using type because it's localized, class is not *)
				(* do this recursive, now with the contents of this folder *)
				set aliasList to every item of finderItem (* use of "every file" would result in a list only containing the files *)
				if ((count of aliasList) is greater than 0) then
					repeat with oneItem in items of aliasList
						tell me to createChecksums(oneItem, dryRun)
					end repeat
				end if
			else
				(* Don't check checksums of checksums! *)
				if (theItemName does not end with ".md5") then
					set fileCount to fileCount + 1
					(* If dry-running, only increment file counter *)
					if (dryRun is not true) then
						(* Check for checksum file *)
						set checkSum to ""
						set checksumFileName to theItemName & ".md5"
						set getChecksumFileContentCommand to "cd \"" & itemContainerPath & "\" && cat \"" & checksumFileName & "\""
						try
							tell AppleScript to set checkSum to do shell script getChecksumFileContentCommand
						on error (e)
							log "Error: " & e (* Probably the md5 file is missing or has a fucked-up name *)
						end try
						log "checksum from .md5-file: " & checkSum
						(* Is the read checksum valid? *)
						if (length of checkSum is less than 5) then
							set logMessage to logMessage & dialogChecksumFileMissingMessage01 & checksumFileName & dialogChecksumFileMissingMessage02 & return
						else
							(* Create checksum for file *)
							set cmd to "cd \"" & itemContainerPath & "\" && md5 -q \"" & theItemName & "\" | head -1"
							#		log "Creating md5 checksum with this command: " & cmd
							tell AppleScript to set theItemsChecksum to do shell script cmd
							log "checksum from file: " & theItemsChecksum
							(* Compare checksum *)
							if (theItemsChecksum is not equal to checkSum) then
								set logMessage to logMessage & dialogAttentionWord & dialogChecksumMismatchMessage01 & theItemName & dialogChecksumMismatchMessage02 & " (" & theItemsChecksum & "/" & checkSum & ")" & return & return
								log "Checksum does not match: " & theItemName
							end if
						end if
					end if (* if dryRun *)
				end if (* if not md5 checksum file *)
			end if (* if file *)
		end tell (* tell finder *)
	on error (e)
		set logMessage to logMessage & "Error: " & e & return
	end try
end checkChecksums

on systemLanguage()
	set cmd to "defaults read NSGlobalDomain AppleLanguages | grep -v \"^(\" | grep -v \"^)\" | head -1 | cut -d, -f1 | awk '{printf $1}'"
	
	try
		tell AppleScript to set languageCode to do shell script cmd
	on error (e)
		log "WTF? Cannot determine language code."
	end try
	
	if (languageCode is "de") then
		log "Deutsch"
	else if (languageCode is "en") then
		log "English"
	else
		log "WTF?"
	end if
	
	return languageCode
end systemLanguage





































































































(* nothing here *)