(*

  File Merge.scpt
  
  (c) by Heiko Kretschmer
  
 This Script merges some files into one big file, probably because you split them with "File Split.scpt"?
   
  This Script is free of charge. Use it as you wish. Don't blame me if you or the script screws up.
  As usual, feel free to use the code for your own projects.

*)
property pBadSelectionMessage : "Please select more than one files in Finder."
property pBadSelectionButton : "Allright, I do whatever you say!"
property pErrorMessage : "Something went wrong:"
property pErrorButton : "Duh!"

(* Unmain (http://mindprod.com/jgloss/unmain.html) :-) *)
on run (argv)
	(* Tell finder to get me the selected files or folders *)
	try
		tell application "Finder"
			activate
			set selectedItems to selection
			(* Only accept more than one selected file *)
			if ((count of selectedItems) is greater than 1) then
				(* Check if only files were selected, hopefully only files of one folder... *)
				set theItemsContainer to (container of first item of selectedItems) as alias
				set theItemsContainerPath to POSIX path of theItemsContainer
				set selectionContainsOnlyFiles to true
				repeat with theItem in selectedItems
					set theItemClass to class of theItem (* not using 'type' because it's localized, 'class' is not *)
					if (theItemClass is folder) then
						(* Won't process folders *)
						set selectionContainsOnlyFiles to false
					end if
				end repeat
				if selectionContainsOnlyFiles is true then
					
					(* Convert from file-list to string-list and add quotes *)
					set filenameList to {}
					repeat with oneFile in selectedItems
						set filenameList to filenameList & ("\"" & name of oneFile & "\"")
					end repeat
					
					(* Since we can't be sure the files are in correct order, sort them! *)
					tell me to set sortedList to quickSort(filenameList)
					
					(* Convert list of strings to one string *)
					set oldDelim to AppleScript's text item delimiters
					set AppleScript's text item delimiters to " "
					set sortedListString to sortedList as string
					set AppleScript's text item delimiters to oldDelim
					log sortedListString
					
					(* Guess the original file name *)
					tell me to set mergedFileName to cut(first item in sortedList, ".Part_", 1) & "\""
					
					(* Concatenate files *)
					set cmd to "cd \"" & theItemsContainerPath & "\" && test ! -f " & mergedFileName & " && cat " & sortedListString & " > " & mergedFileName
					log cmd
					do shell script cmd
					
				else
					display dialog pBadSelectionMessage buttons pBadSelectionButton
				end if
			else
				display dialog pBadSelectionMessage buttons pBadSelectionButton
			end if
		end tell
	on error (e)
		tell application "Finder"
			activate
			display dialog pErrorMessage & return & return & e buttons pErrorButton with icon 2
		end tell
	end try
end run

(* log cut("foo:bar:gaga", ":", 1) *)
to cut(str, delim, field)
	set oldAST to AppleScript's text item delimiters
	set AppleScript's text item delimiters to delim
	set elements to text items of str
	set AppleScript's text item delimiters to oldAST
	set res to item field of elements
	return res
end cut

to quickSort(theList) (* found at http://macscripter.net/viewtopic.php?id=24766 *)
	--public routine, called from your script
	script bs
		property alist : theList
		
		on Qsort(leftIndex, rightIndex)
			--private routine called by quickSort. 
			--do not call from your script!
			if rightIndex > leftIndex then
				set pivot to ((rightIndex - leftIndex) div 2) + leftIndex
				set newPivot to Qpartition(leftIndex, rightIndex, pivot)
				set theList to Qsort(leftIndex, newPivot - 1)
				set theList to Qsort(newPivot + 1, rightIndex)
			end if
			
		end Qsort
		
		on Qpartition(leftIndex, rightIndex, pivot)
			--private routine called by quickSort. 
			--do not call from your script!
			set pivotValue to item pivot of bs's alist
			set temp to item pivot of bs's alist
			set item pivot of bs's alist to item rightIndex of bs's alist
			set item rightIndex of bs's alist to temp
			set tempIndex to leftIndex
			repeat with pointer from leftIndex to (rightIndex - 1)
				if item pointer of bs's alist  pivotValue then
					set temp to item pointer of bs's alist
					set item pointer of bs's alist to item tempIndex of bs's alist
					set item tempIndex of bs's alist to temp
					set tempIndex to tempIndex + 1
				end if
			end repeat
			set temp to item rightIndex of bs's alist
			set item rightIndex of bs's alist to item tempIndex of bs's alist
			set item tempIndex of bs's alist to temp
			
			return tempIndex
		end Qpartition
		
	end script
	
	if length of bs's alist > 1 then bs's Qsort(1, length of bs's alist)
	return bs's alist
end quickSort