(*

  File Split.scpt
  
  (c) by Heiko Kretschmer
  
 This Script splits one big file in some smaller ones. Uses "split" as known in various Unixes and Linuxes.
  
  This Script is free of charge. Use it as you wish. Don't blame me if you or the script screws up.
  As usual, feel free to use the code for your own projects.

*)
property pBadSelectionMessage : "Please select (only) one file in Finder."
property pBadSelectionButton : "Allright, I do whatever you say!"
property pChooseSizeMessage : "Please choose a maxium file size for"
property pChooseSizeButton : "Select"
property pErrorMessage : "Something went wrong:"
property pErrorButton : "Duh!"

(* Unmain (http://mindprod.com/jgloss/unmain.html) :-) *)
on run (argv)
	(* Tell finder to get me the selected files or folders *)
	try
		tell application "Finder"
			activate
			set selectedItems to selection
			(* Only accept one selected file *)
			if ((count of selectedItems) is 1) then
				set theItem to first item in selectedItems
				set theItemClass to class of theItem
				log theItemClass
				set theItemName to name of theItem
				set theItemContainer to (container of theItem) as alias
				set itemPath to POSIX path of (theItem as alias)
				set itemContainerPath to POSIX path of theItemContainer
				if (theItemClass is folder) then (* not using 'type' because it's localized, 'class' is not *)
					(* Won't process folders *)
					display dialog pBadSelectionMessage buttons pBadSelectionButton
					#				set l to length of itemPath
					#				set itemPath to text 1 thru (l - 1) of itemPath
					#				log "shortened: " & itemPath
				else
					(* X' user for file size, see See http://macscripter.net/viewtopic.php?id=10988 *)
					set sizesList to {"1MB", "10MB", "100MB", "500MB", "1000MB"}
					choose from list sizesList with prompt pChooseSizeMessage & " \"" & theItemName & " \":"
					if (result as string is not "false") then (* result is either false or the selected listitem as string *)
						(* Translate from readable to 'split'-usable sizes *)
						set listchoice to result as text
						#tell me to set maxSize to removeTrailingCharacters(listchoice, 1)
						tell me to set maxSize to changeString of listchoice from "MB" to "m"
						(* Process file *)
						set cmd to "cd \"" & itemContainerPath & "\" && split -b " & maxSize & " \"" & theItemName & "\"" & " \"" & theItemName & ".Part_\""
						log cmd
						do shell script cmd
					end if
				end if
			else
				display dialog pBadSelectionMessage buttons pBadSelectionButton
			end if
		end tell
	on error (e)
		tell application "Finder"
			activate
			display dialog pErrorMessage & return & return & e buttons pErrorButton with icon 2
		end tell
	end try
end run

to removeTrailingCharacters(theItem, num)
	if the length of theItem > 0 then
		#	if (the character 1 of theItem is " ") then set theItem to characters 2 thru end of theItem as string
		log (length of theItem)
		set theItem to characters 1 thru ((length of theItem) - num) of theItem as string
	end if
	return theItem
end removeTrailingCharacters

to removeTrailingSpaces(theItem)
	if the length of theItem > 0 then
		#	if (the character 1 of theItem is " ") then set theItem to characters 2 thru end of theItem as string
		log (length of theItem)
		if (the last character of theItem is " ") then
			set theItem to characters 1 thru ((length of theItem) - 1) of theItem as string
		end if
	end if
end removeTrailingSpaces

to changeString of t from s to r
	set d to text item delimiters
	set text item delimiters to s
	set t to t's text items
	set text item delimiters to r
	tell t to set t to beginning & ({""} & rest)
	set text item delimiters to d
	t
end changeString